@@ -18,7 +18,8 @@ import traceback
 
 from typing import Union
 
-from alex.alex import checkpoint, const, util, validation
+from alex.alex import checkpoint, const, util, validation, registry
+from alex.alex.logger import logger
 
 
 strs = list(string.ascii_lowercase)
@@ -58,17 +59,17 @@ def alex_reader(user_defined):
         component = clone(component)
         hyperparams = dict() if const.HYPERPARAMS not in component else clone(component[const.HYPERPARAMS])
         component_type = component[const.TYPE]
-        if component_type in const.PARAMS:
-            for param in const.PARAMS[component_type]:
+        if component_type in registry.PARAMS:
+            for param in registry.PARAMS[component_type]:
                 if param not in hyperparams:
                     hyperparams[param] = dict()
                 if const.DTYPE not in hyperparams[param]:
-                    hyperparams[param][const.DTYPE] = const.DEFAULT_DTYPE
+                    hyperparams[param][const.DTYPE] = registry.DEFAULT_DTYPE
         component[const.HYPERPARAMS] = hyperparams
         return component
 
     if "default" not in user_defined:
-        default = {"dtype": const.DEFAULT_DTYPE}
+        default = {"dtype": registry.DEFAULT_DTYPE}
     else:
         default = user_defined["default"]
 
@@ -77,6 +78,7 @@ def alex_reader(user_defined):
               const.INPUTS: _read_inputs,
               const.HYPERPARAMS: _read_hyperparams,
               "visible": lambda x: {**{"visible": True}, **x} if "visible" not in x else clone(x),
+              "trainable": lambda x: {**{"trainable": None}, **x} if "trainable" not in x else clone(x),
               "dtype": lambda x: {**{"dtype": default["dtype"]}, **x} if "dtype" not in x else clone(x)}
     return reader
 
@@ -93,8 +95,8 @@ def clone(component):
 
 def recipe_defined(rtype):
     # TODO: temporary. Use types instead
-    return rtype not in list(const.DET_COMPONENTS_NO_HYPE.keys()) + [const.DATA,
-                                                                     const.LABEL]
+    return rtype not in list(registry.STATELESS_INGREDIENTS_WITHOUT_HYPE.keys()) + [const.DATA,
+                                                                                    const.LABEL]
 
 # FIXME
 def _resolve_hyperparams(hyperparams):
@@ -115,8 +117,8 @@ def _parse_ingredient_hyperparams(hyperparams):
     return hyperparams
 
 
-def _config_to_inter_graph(user_defined, reader, block, parent_type="root", config=None, recipe=None):
-    if config is None:
+def _config_to_inter_graph(user_defined, reader, block, parent_type="root", config=None, recipe=None, inputs=None):
+    if config is None and "params_network" not in user_defined:
         config = user_defined
         recipe = const.ROOT
         component = dict()
@@ -134,28 +136,37 @@ def _config_to_inter_graph(user_defined, reader, block, parent_type="root", conf
                                                                        recipe=block))
 
     else:
+        if config is None:
+            config = deepcopy(user_defined)
         # If configuration is not an ingredient
         if const.PARAMS_NETWORK in config:
             component = dict()
             component[const.TYPE] = recipe
             if const.PARAMS_NETWORK not in user_defined:
                 component = fill_mandatory(clone(user_defined), reader)
+
             component[const.HYPERPARAMS] = []
             component["block"] = block
             component[const.SCOPE] = parent_type
-            parent_type = recipe
             for subconfig in config[const.PARAMS_NETWORK]:
                 if recipe_defined(subconfig[const.TYPE]):
                     subconfig_def = util.read_yaml(os.path.join(const.COMPONENT_BASE_PATH,
                                                                 subconfig[const.TYPE] + const.YAML))
                 else:
                     subconfig_def = {}
+
+                if "inputs" in subconfig:
+                    _inputs = subconfig["inputs"]
+                else:
+                    _inputs = None
                 component[const.HYPERPARAMS].append(_config_to_inter_graph(subconfig,
                                                                            reader,
                                                                            block,
-                                                                           parent_type,
+                                                                           recipe,
                                                                            subconfig_def,
-                                                                           subconfig[const.TYPE]))
+                                                                           subconfig[const.TYPE], _inputs))
+            if inputs is not None:
+                component[const.HYPERPARAMS][0]["inputs"] = inputs
         else:
             component = fill_mandatory(clone(user_defined), reader)
             hyperparams = clone(config)
@@ -197,6 +208,7 @@ def _get_node(component):
             graph[const.META][m] = component[m]
     graph[const.META][const.NAME] = None
     graph[const.META]["visible"] = component["visible"]
+    graph[const.META]["trainable"] = component["trainable"]
     graph[const.META]["block"] = component["block"]
     graph[const.META]["dtype"] = component["dtype"]
     graph[const.META][const.SCOPE] = component[const.SCOPE] if const.SCOPE in component else component[const.TYPE]
@@ -276,13 +288,19 @@ def eval_repeat(times, components):
 
 
 def _is_block(scope):
-    return scope in const.BLOCKS
+    return scope in registry.BLOCKS
 
 
 def eval_name(name, components):
     components = clone(components)
     top_name = _get_name(name)
     __scope = components[0][const.META][const.SCOPE]
+    __type =  components[0][const.META][const.TYPE]
+    for component in components:
+        if "dir" in component[const.META] and component[const.META]["dir"] != "":
+            continue
+        __scope = component[const.META][const.SCOPE]
+        break
     if __scope == const.ROOT:
         scope = ""
     else:
@@ -316,6 +334,13 @@ def eval_name(name, components):
                 else:  # user defined name in recipe
                     __name = "%s/%s" % (_name, __name)
         component[const.META][const.NAME] = __name
+        component[const.META]["dir"] = "/".join(__name.split("/")[:-1])
+
+        if "recipes" not in component[const.META]:
+            component[const.META]["recipes"] = []
+        component[const.META]["recipes"].append(scope)
+        if len(component[const.META]["recipes"]) > len(__name.split("/")[:-1]):
+            component[const.META]["recipes"] = component[const.META]["recipes"][1:]
     return components
 
 
@@ -345,8 +370,13 @@ def eval_inputs(inputs, components):
                     for _b, _base in enumerate(component[const.META][const.INPUTS]):
                         if isinstance(_base, dict):
                             continue
-                        if prev_dir in _base:
-                            _in = name_map[_base]
+                        # TO_BE_TESTED
+                        # current_scope = component[const.META]["scope"]
+                        # if current_scope in registry.BLOCKS:
+                        #     if "/" not in _base:
+                        #         _in = "%s/%s" % (current_scope, _base)
+                        if _base.split("/")[0] in registry.BLOCKS:
+                            _in = _base
                         else:
                             base_name = _base.split("/")[-1]
                             _in = prev_dir + base_name
@@ -461,7 +491,8 @@ def connect_to(components, current_component_type, current_component_hyperparams
         return _component["meta"]["type"]
 
     if scope != "all" and (scope != "current"):
-        raise Exception("scope has to be 'current' or 'all'!")
+        logger.error("Scope has to be 'current' or 'all'!")
+        raise Exception
     if current_component_hyperparams is None:
         current_component_hyperparams = components[-1]["value"]["hyperparams"]
     current_component_name_scope = _get_name_scope(components[-1])
@@ -561,7 +592,7 @@ def global_update(components):
 
 ########### Params_Training ###############
 def _user_fn(fn, kwargs):
-    return lambda x: const.USER_FNS[fn](x, **kwargs)
+    return lambda x: registry.USER_FNS[fn](x, **kwargs)
 
 
 ######################################### Different views:
@@ -586,16 +617,17 @@ def draw_graph(graph_list, level=2, graph_path='example.png', show="name"):
             label = name.split("/")[-1]# .split("_")[0]
         else:
             if show not in component["meta"] or show=="type":
-                raise Exception("Lable '%s' is not available" % show)
+                logger.error("Lable '%s' is not available" % show)
+                raise Exception
             label = str(component["meta"][show])
         node.set_label(label)
         graph.add_node(node)
 
         _type = component[const.META][const.TYPE]
-        if not _type in const.INPUT_TYPES:
+        if not _type in registry.DATA_BLOCK:
             inputs = component[const.META][const.INPUTS]
             for _input in inputs:
-                if _type not in const.REGULARIZERS:
+                if _type not in registry.REGULARIZERS:
                     _input = "/".join(_input.split("/")[:level])
                 _edge = pydot.Edge(_input, name)
                 graph.add_edge(_edge)
@@ -606,73 +638,52 @@ def draw_graph(graph_list, level=2, graph_path='example.png', show="name"):
 
 def _get_dir_name(path, level):
     _dir = path.split("/")
-    scope = path.replace(_dir[-1], "")
-    scope = "/".join(scope.split("/")[0:level-1]) + "/"
+    scope = path.replace("/%s"%_dir[-1], "")
+    scope = "/".join(scope.split("/")[0:level])
     return scope, _dir[-1]
 
 
-def list_to_graph(components, parent_level=1, parent_scope=""):
-    network = dict()
-    _subgraph = OrderedDict()
-    network[const.LEVEL] = parent_level-1
-    network[const.META] = dict()
-    # TODO: remove the following info (cached in meta)
-    network[const.INPUTS] = None
-    network["visible"] = True
-    network[const.TYPE] = const.ROOT
+def _register_component(component, level):
+    subgraph = dict()
+    path = component[const.META][const.NAME]
+    subgraph[const.SUBGRAPH] = component[const.META][const.NAME]
+    subgraph[const.TYPE] = component[const.META][const.TYPE]
+    subgraph[const.META] = component[const.META]
+    subgraph[const.LEVEL] = level
+    subgraph[const.HYPERPARAMS] = component[const.VALUE][const.HYPERPARAMS]
+    return subgraph
+
+
+def _register_graph(level, node_type="root", meta={"inputs": None,
+                                                   "name": "root"}):
+    graph = dict()
+    graph[const.SUBGRAPH] = OrderedDict()
+    graph[const.LEVEL] = level
+    graph[const.HYPERPARAMS] = dict()
+    graph[const.TYPE] = node_type
+    graph[const.META] = meta
+    return graph
+
+
+def list_to_graph(components):
+    graph = _register_graph(0)
     i = 0
-    while i < len(components):
-        component = components[i]
-        hyperparams = component[const.VALUE][const.HYPERPARAMS]
+
+    for component in components:
         path = component[const.META][const.NAME]
         _dir = path.split("/")
-        level = len(_dir)
-        scope, _name = _get_dir_name(path, level)
-        inputs = component[const.META][const.INPUTS]
-        if parent_scope != scope:
-            if level > parent_level:
-                _components = []
-                in_scope = False
-                in_scope_previous = False
-                for _component in components[i:]:
-                    _path = _component[const.META][const.NAME]
-                    __scope, __ = _get_dir_name(_path, level)
-                    if scope == __scope:
-                        in_scope = True
-                        _components.append(_component)
-                        i += 1
-                    else:
-                        in_scope = False
-                    if in_scope_previous and not (in_scope):
-                        break
-                    in_scope_previous = in_scope
-                _network = list_to_graph(_components, level, scope)
-                _network[const.INPUTS] = inputs
-                _network[const.HYPERPARAMS] = dict() # hyperparams for recipes
-                _network[const.TYPE] = component[const.META][const.SCOPE]
-                # _network["visible"] = component[const.META]["visible"]
-                # _network["block"] = component[const.META]["block"]
-                _network[const.META] = component[const.META]
-                _subgraph[scope] = _network
-            else:
-                break
-        else:
-            _network = dict()
-            _network[const.SUBGRAPH] = component[const.META][const.NAME]
-            _network[const.TYPE] = component[const.META][const.TYPE]
-            _network[const.META] = component[const.META]
-            # _network["dtype"] = component[const.META]["dtype"]
-            # _network["visible"] = component[const.META]["visible"]
-            # _network["block"] = component[const.META]["block"]
-            _network[const.INPUTS] = inputs
-            _network[const.LEVEL] = level
-            _network[const.HYPERPARAMS] = hyperparams
-
-            _subgraph[path] = _network
-            i += 1
-
-    network[const.SUBGRAPH] = _subgraph
-    return network
+        levels = len(_dir)
+        _graph = graph
+        for _level in range(1, levels+1):
+            scope = "/".join(_dir[0:_level])
+            if _level == levels: # is leaf
+                _graph["subgraph"][scope] = _register_component(component, _level)
+            elif scope not in _graph["subgraph"]:
+                _graph["subgraph"][scope] = _register_graph(_level,
+                                                            component["meta"]["recipes"][levels-_level-1],
+                                                            component["meta"])
+            _graph = _graph["subgraph"][scope]
+    return graph
 
 
 def draw_hyperparam_tree(graph,
@@ -688,17 +699,17 @@ def draw_hyperparam_tree(graph,
         return not isinstance(graph, Iterable) or (isinstance(graph, str))
 
     def _add_node(parent_node, label, name, graph):
-        if label in const.COMPONENTS:
-            color = colors["COMPONENTS"]
-        elif label in const.DET_COMPONENTS_HYPE:
-            color = colors["DET_COMPONENTS_HYPE"]
-        elif label in const.DET_COMPONENTS_NO_HYPE:
-            color = colors["DET_COMPONENTS_NO_HYPE"]
-        elif label in const.INFERENCE:
+        if label in registry.STATEFUL_INGREDIENTS:
+            color = colors["STATEFUL_INGREDIENTS"]
+        elif label in registry.STATELESS_INGREDIENTS_WITH_HYPE:
+            color = colors["STATELESS_INGREDIENTS_WITH_HYPE"]
+        elif label in registry.STATELESS_INGREDIENTS_WITHOUT_HYPE:
+            color = colors["STATELESS_INGREDIENTS_WITHOUT_HYPE"]
+        elif label in registry.INFERENCES:
             color = colors["INFERENCE"]
-        elif label in const.ALL_TRAINABLE_PARAMS:
+        elif label in registry.ALL_TRAINABLE_PARAMS:
             color = colors["TRAINABLE_PARAMS"]
-        elif label in const.ALL_OTHER_PARAMS:
+        elif label in registry.ALL_OTHER_PARAMS:
             color = colors["NONTRAINABLE_PARAMS"]
         else:
             color = "white"
@@ -735,8 +746,9 @@ def draw_hyperparam_tree(graph,
                 if _is_simple(_node):
                     label = str(_node) # node is simple
                 else:
-                    print(_node)
-                    raise Exception("Unrecognized node type")
+                    pprint(_node)
+                    logger.error("Unrecognized node type")
+                    raise Exception
 
                 name = "%s/%s" % (parent_node, label)
                 if isinstance(_graph, list):
@@ -784,9 +796,9 @@ def draw_hyperparam_tree(graph,
 
         return graph
 
-    colors = {"COMPONENTS": "red",
-              "DET_COMPONENTS_HYPE": "green",
-              "DET_COMPONENTS_NO_HYPE": "yellow",
+    colors = {"STATEFUL_INGREDIENTS": "red",
+              "STATELESS_INGREDIENTS_WITH_HYPE": "green",
+              "STATELESS_INGREDIENTS_WITHOUT_HYPE": "yellow",
               "INFERENCE": "cyan",
               "NONTRAINABLE_PARAMS": "#8ecae6",
               "TRAINABLE_PARAMS": "#ffafcc"}
@@ -865,9 +877,8 @@ def annotate(components, lazy=True):
 
             inputs = component["meta"]["inputs"]
         except Exception:
-            print("------------- Component %s annotation failed ----------" % (name))
-            traceback.print_exc()
-            raise
+            logger.error("Component %s annotation failed" % (name))
+            raise Exception
         components[name] = component
     return list(components.values())
 
@@ -922,13 +933,13 @@ def parse(yml_file, return_dict=False, lazy=True):
         graph:list = eval_ast_to_graph(ast)
         # Step 4: second pass: same data structure; populate inputs
         graph:list = global_update(graph)
-        # if config_to_type(yml_file) not in const.ALL_RECIPES:
+        # if config_to_type(yml_file) not in const.RECIPE_TYPES:
         graph = annotate(graph, lazy=lazy) # json schema
         # Step 5: load graph and states from checkpoint; check if the structure matches the one defined in the DSL
     except Exception:
-        message = "-------------- Error during parsing configuration %s ----------" % yml_file
-        traceback.print_exc()
-        raise
+        message = "Error during parsing configuration %s" % yml_file
+        logger.error(message)
+        raise Exception
     if return_dict:
         graph = list_to_dict(graph)
     return graph
