@@ -31,7 +31,7 @@ from rope.refactor.inline import create_inline
 import warnings
 import traceback
 
-from alex.alex import core, const, node_interface, checkpoint, util
+from alex.alex import core, const, node_interface, checkpoint, util, registry
 from alex.alex.checkpoint import Checkpoint
 from alex.alex.annotator_interface import Annotator
 from alex.engine import ns_alex, ns_tf, ns_pytorch
@@ -42,6 +42,7 @@ NAMESPACES = {"tf": ns_tf,
               "pytorch": ns_pytorch}
 
 
+# Tags: value, function, identifier
 VALUE = TypeVar('VALUE', str, list, int, float)
 FUNCTION = TypeVar('FUNCTION')
 IDENTIFIER = TypeVar('IDENTIFIER')
@@ -49,17 +50,18 @@ IDENTIFIER = TypeVar('IDENTIFIER')
 runtime_keys = ["channels", "batch_size"]
 
 
-def get_node_type(node):
+def get_node_tag(node):
     if node["value"] in ["input_shape"]:
-        return IDENTIFIER # FIXME: temporary for backward compatibility
-    elif node["value"] in const.FNS + ["shape"] + const.ALL_PARAMS_LIST:
-        return FUNCTION
-    elif len(node["children"])==0:
-        return VALUE
+        tag = IDENTIFIER # FIXME: temporary for backward compatibility
+    elif node["value"] in registry.TAG_FUNCTIONS:
+        tag = FUNCTION
+    elif len(node["children"])==0: # is leaf
+        tag = VALUE
     elif len(node["children"]) == len(node["descendants"]):
-        return IDENTIFIER
+        tag = IDENTIFIER
     else:
-        return IDENTIFIER
+        tag = IDENTIFIER
+    return tag
 
 
 def _name_strs_to_names(name_strs):
@@ -83,9 +85,7 @@ def _parse_str(value, node=None, literal=True):
         elif value == "batch_size":
             parsed = "batch_size"
             literal = False
-    if value in const.ALL_COMPONENTS:
-        parsed = "%s()" % value
-    elif literal:
+    if literal:
         parsed = "'%s'" % value
     else:
         parsed = "%s" % value
@@ -118,7 +118,7 @@ def cache_fn(node: dict,
              args: dict,
              return_symbol: str,
              name: str,
-             node_type,
+             node_tag,
              inline: bool = False,
              extra: Union[str, None]=None):
     """
@@ -132,11 +132,11 @@ def cache_fn(node: dict,
     for _arg in args:
         arg = args[_arg]
         if arg["value"] is not None:
-            if arg["type"] == VALUE:
+            if arg["tag"] == VALUE:
                 _arg_str = value_to_kwarg_str("",
                                               arg["value"])
-            elif arg["type"] == IDENTIFIER:
-                if node_type == FUNCTION:
+            elif arg["tag"] == IDENTIFIER:
+                if node_tag == FUNCTION:
                     if arg["str"] == "":
                         continue
                     _arg_str = "%s=%s" % (arg["key"], arg["str"])
@@ -147,18 +147,18 @@ def cache_fn(node: dict,
             else:
                 if arg["value"] == "":
                     continue
-                if node_type == FUNCTION:
+                if node_tag == FUNCTION:
                     _arg_str = "%s=%s" % (arg["key"], arg["value"])
                 else:
                     _arg_str = arg["value"]
 
             arg_str.append(_arg_str)
 
-    if node_type == IDENTIFIER and len(arg_str)>1:
+    if node_tag == IDENTIFIER and len(arg_str)>1:
         arg_str = "[%s]" % ", ".join(arg_str)
     else:
         arg_str = ", ".join(arg_str)
-    if node_type == FUNCTION:
+    if node_tag == FUNCTION:
         if len(arg_str) != "":
             arg_str = "(%s)" % (arg_str)
         code_str = "%s%s" % (fn,
@@ -168,7 +168,7 @@ def cache_fn(node: dict,
     node["code"] = {"fn": fn,
                     "args": args,
                     "return_symbol": return_symbol,
-                    "type": node_type,
+                    "tag": node_tag,
                     "name": name,
                     "str": code_str,
                     "inline": inline,
@@ -194,25 +194,25 @@ class Ingredient(param_count.Ingredient):
         args[const.ALEX_ARG_INPUTS] = {"key": const.ALEX_ARG_INPUTS,
                                        "value": inputs,
                                        "ref": inputs,
-                                       "type": IDENTIFIER,
+                                       "tag": IDENTIFIER,
                                        "str": value_to_kwarg_str("", inputs,
                                                                  literal=False)}
         args[const.ALEX_ARG_NAME] = {"key": const.ALEX_ARG_NAME,
                                      "value": _name_strs_to_names(node["name"]),
                                      "ref": None,
-                                     "type": IDENTIFIER,
+                                     "tag": IDENTIFIER,
                                      "str": "'%s'" % (node["name"])}
         if node["input_nodes"] is not None:
             input_nodes = annotated[node["input_nodes"][0]]
             args["shape"] = {"key": "shape",
                              "value": input_nodes,
                              "ref": None,
-                             "type": IDENTIFIER,
+                             "tag": IDENTIFIER,
                              "str": "%s" % str(input_nodes["shape"])}
         args["dtype"] = {"key": "dtype",
                          "value": node["dtype"],
                          "ref": None,
-                         "type": IDENTIFIER,
+                         "tag": IDENTIFIER,
                          "str": "'%s'" % (node["dtype"])}
         return_symbol = _name_strs_to_names(node["name"])
 
@@ -221,10 +221,10 @@ class Ingredient(param_count.Ingredient):
                 continue
             _arg = child["code"]["args"]
             for _key in _arg:
-                if _arg[_key]["type"] == IDENTIFIER:
+                if _arg[_key]["tag"] == IDENTIFIER:
                     args = {**args, **{_key: _arg[_key]}}
-            if child["code"]["type"] != VALUE:
-                if child["value"] in const.ALL_PARAMS:
+            if child["code"]["tag"] != VALUE:
+                if child["value"] in registry.ALL_PARAMS:
                     _name = "%s/%s" % (core.get_parent(child["name"],
                                                        annotated,
                                                        "name"),
@@ -234,13 +234,13 @@ class Ingredient(param_count.Ingredient):
                             **{child["value"]: {"key": child["value"],
                                                 "value": "trainable_params['%s']" % _name,
                                                 "ref": child["name"],
-                                                "type": child["code"]["type"],
+                                                "tag": child["code"]["tag"],
                                                 "str": child["code"]["str"]}}}
                 else:
                     _child = {"key": child["value"],
                               "value": child["code"]["return_symbol"],
                               "ref": child["name"],
-                              "type": child["code"]["type"],
+                              "tag": child["code"]["tag"],
                               "str": child["code"]["str"]}
                     if child["value"]=="decay":
                         if engine == "tf":
@@ -269,29 +269,29 @@ class Regularizer(param_count.Ingredient):
         args[const.ALEX_ARG_INPUTS] = {"key": const.ALEX_ARG_INPUTS,
                                        "value": node["meta"]["position"]["inputs"],
                                        "ref": None,
-                                       "type": IDENTIFIER,
+                                       "tag": IDENTIFIER,
                                        "str": value_to_kwarg_str("",
                                                                  node["meta"]["position"]["inputs"],
                                                                  literal=True)}
         args[const.ALEX_ARG_NAME] = {"key": const.ALEX_ARG_NAME,
                                      "value": _name_strs_to_names(node["name"]),
                                      "ref": None,
-                                     "type": IDENTIFIER,
+                                     "tag": IDENTIFIER,
                                      "str": "'%s'" % (node["name"])}
         args["dtype"] = {"key": "dtype",
                          "value": node["dtype"],
                          "ref": None,
-                         "type": IDENTIFIER,
+                         "tag": IDENTIFIER,
                          "str": "'%s'" % (node["dtype"])}
         return_symbol = _name_strs_to_names(node["name"])
 
         for child in children:
             _arg = child["code"]["args"]
             for _key in _arg:
-                if _arg[_key]["type"] == IDENTIFIER:
+                if _arg[_key]["tag"] == IDENTIFIER:
                     args = {**args, **{_key: _arg[_key]}}
-            if child["code"]["type"] != VALUE:
-                if child["value"] in const.ALL_PARAMS:
+            if child["code"]["tag"] != VALUE:
+                if child["value"] in registry.ALL_PARAMS:
                     _name = "%s/%s" % (core.get_parent(child["name"],
                                                        annotated,
                                                        "name"),
@@ -301,14 +301,14 @@ class Regularizer(param_count.Ingredient):
                             **{child["value"]: {"key": child["value"],
                                                 "value": "trainable_params['%s']" % _name,
                                                 "ref": child["name"],
-                                                "type": child["code"]["type"],
+                                                "tag": child["code"]["tag"],
                                                 "str": child["code"]["str"]}}}
                 else:
                     args = {**args,
                             **{child["value"]: {"key": child["value"],
                                                 "value": child["code"]["return_symbol"],
                                                 "ref": child["name"],
-                                                "type": child["code"]["type"],
+                                                "tag": child["code"]["tag"],
                                                 "str": child["code"]["str"]}}}
         node = cache_fn(node,
                         node["value"],
@@ -327,7 +327,7 @@ class Recipe(param_count.Recipe):
     def generate_code(self, node, annotated, engine):
         node = deepcopy(node)
         node["code"] = dict()
-        node["code"]["type"] = IDENTIFIER
+        node["code"]["tag"] = IDENTIFIER
         return node
 
 
@@ -342,18 +342,18 @@ class Function(param_count.Hyperparam):
         return_symbol = _name_strs_to_names(node["name"])
         children = core.get_children(node["name"], annotated)
         for child in children:
-            _node_type = child["code"]["type"]
-            if _node_type == VALUE:
+            _node_tag = child["code"]["tag"]
+            if _node_tag == VALUE:
                 continue
-            elif _node_type == IDENTIFIER:
+            elif _node_tag == IDENTIFIER:
                 _value = child["code"]["str"]
-            elif _node_type == FUNCTION:
+            elif _node_tag == FUNCTION:
                 _value = child["code"]["return_symbol"]
             args = {**args,
                     **{child["value"]: {"key": child["value"],
                                         "value": _value,
                                         "ref": child["name"],
-                                        "type": child["code"]["type"],
+                                        "tag": child["code"]["tag"],
                                         "str": child["code"]["str"]}}}
         node = cache_fn(node,
                         node["value"],
@@ -373,26 +373,33 @@ class Value(param_count.Hyperparam):
         if "padding" in node["parent"] and engine=="pytorch":
             ingredient = annotated[node["meta"]["position"]["component"]]
             if ingredient["value"] in ["conv", "max_pool2d"]:
+                # FIXME:
                 if node["value"] == "SAME":
-                    _value = "same"
-                    # _value = [1, 1]
-                    # inputs = annotated[ingredient["meta"]["position"]["inputs"][0]]
-                    # input_shape = inputs["shape"]
-                    # h, w = input_shape[0], input_shape[1]
-                    strides = ingredient["meta"]["hyperparams"]["strides"]
-                    if strides[0]!=1 or strides[1]!=1:
-                        raise Exception("Pytorch does not support strided ops  for padding='same'")
-                    # if ingredient["value"] == "conv":
-                    #     k_h = ingredient["meta"]["hyperparams"]["filters"]["shape"]["kernel_size_h"]
-                    #     k_w = ingredient["meta"]["hyperparams"]["filters"]["shape"]["kernel_size_w"]
-                    # else:
-                    #     [k_h, k_w] = ingredient["meta"]["hyperparams"]["window_shape"]
-                    # padding_h_min = floor(((ingredient["shape"][0]-1)*strides[0] - h + k_h)/2)
-                    # padding_w_min = floor(((ingredient["shape"][1]-1)*strides[1] - w + k_w)/2)
-                    # _value = [padding_h_min, padding_w_min]
+                    _value = [1, 1]
+
                 elif node["value"] == "VALID":
-                    # _value = [0, 0]
-                    _value = "valid"
+                    _value = [0, 0]
+
+                # "valid" and "same" are only available for pytorch > 1.9
+                # if node["value"] == "SAME":
+                #     _value = "same"
+                #     # _value = [1, 1]
+                #     # inputs = annotated[ingredient["meta"]["position"]["inputs"][0]]
+                #     # input_shape = inputs["shape"]
+                #     # h, w = input_shape[0], input_shape[1]
+                #     strides = ingredient["meta"]["hyperparams"]["strides"]
+                #     if strides[0]!=1 or strides[1]!=1:
+                #         raise Exception("Pytorch does not support strided ops  for padding='same'")
+                #     # if ingredient["value"] == "conv":
+                #     #     k_h = ingredient["meta"]["hyperparams"]["filters"]["shape"]["kernel_size_h"]
+                #     #     k_w = ingredient["meta"]["hyperparams"]["filters"]["shape"]["kernel_size_w"]
+                #     # else:
+                #     #     [k_h, k_w] = ingredient["meta"]["hyperparams"]["window_shape"]
+                #     # padding_h_min = floor(((ingredient["shape"][0]-1)*strides[0] - h + k_h)/2)
+                #     # padding_w_min = floor(((ingredient["shape"][1]-1)*strides[1] - w + k_w)/2)
+                #     # _value = [padding_h_min, padding_w_min]
+                # elif node["value"] == "VALID":
+                #     _value = "valid"
         else:
             _value = node["value"]
         node = cache_fn(node=deepcopy(node),
@@ -400,7 +407,7 @@ class Value(param_count.Hyperparam):
                         args={},
                         return_symbol=_value,
                         name="",
-                        node_type=VALUE)
+                        node_tag=VALUE)
         return node
 
 
@@ -408,8 +415,7 @@ class Shape(param_count.Hyperparam):
 
     def generate_code(self, node, annotated, engine):
         node = deepcopy(node)
-        node_type = get_node_type(node)
-        # FIXME: refactor dispatch; for now it's experimental
+        node_tag = get_node_tag(node)
         args = dict()
         fn = "%s_%s_shape" % (node["ancestor"]["value"],
                               core.get_parent(node["name"],
@@ -419,25 +425,25 @@ class Shape(param_count.Hyperparam):
 
         children = core.get_children(node["name"], annotated)
         for child in children:
-            _node_type = child["code"]["type"]
-            if _node_type == VALUE:
+            _node_tag = child["code"]["tag"]
+            if _node_tag == VALUE:
                 continue
-            elif _node_type == IDENTIFIER:
+            elif _node_tag == IDENTIFIER:
                 _value = child["code"]["str"]
-            elif _node_type == FUNCTION:
+            elif _node_tag == FUNCTION:
                 _value = child["code"]["return_symbol"]
             args = {**args,
                     **{child["value"]: {"key": child["value"],
                                         "value": _value,
                                         "ref": child["name"],
-                                        "type": child["code"]["type"],
+                                        "tag": child["code"]["tag"],
                                         "str": child["code"]["str"]}}}
         node = cache_fn(node,
                         fn,
                         args,
                         return_symbol,
                         "",
-                        node_type,
+                        node_tag,
                         True)
         return node
 
@@ -446,7 +452,7 @@ class Identifier(param_count.Hyperparam):
 
     def generate_code(self, node, annotated, engine):
         node = deepcopy(node)
-        node_type = get_node_type(node)
+        node_tag = get_node_tag(node)
 
         children = core.get_children(node["name"], annotated)
 
@@ -458,14 +464,14 @@ class Identifier(param_count.Hyperparam):
             args[child["value"]] = {"key": child["value"],
                                     "value": child["code"]["return_symbol"],
                                     "ref": child["name"],
-                                    "type": child["code"]["type"],
+                                    "tag": child["code"]["tag"],
                                     "str": child["code"]["str"]}
         node = cache_fn(node=node,
                         fn=node["value"],
                         args=args,
                         return_symbol=_name_strs_to_names(node["name"]),
                         name="",
-                        node_type=node_type)
+                        node_tag=node_tag)
         return node
 
 
@@ -476,43 +482,42 @@ class Initializer(param_count.Hyperparam):
 
     def generate_code(self, node, annotated, engine):
         node = deepcopy(node)
-        node_type = get_node_type(node)
+        node_tag = get_node_tag(node)
 
         args = dict()
 
         param_name = core.get_ancestor_param_node(node, annotated, field="value")
         shape_name = _name_strs_to_names("%s_%s_shape" % (node["ancestor"]["name"],
                                                           param_name))
-
         _name = node["name"]
         return_symbol = _name_strs_to_names(_name)
 
         args["shape"] = {"key": "shape",
                          "value": shape_name,
                          "ref": None,
-                         "type": IDENTIFIER,
+                         "tag": IDENTIFIER,
                          "str": "%s" % shape_name}
 
         args[const.ALEX_ARG_NAME] = {"key": const.ALEX_ARG_NAME,
                                      "value": _name_strs_to_names(_name),
                                      "ref": None,
-                                     "type": IDENTIFIER,
+                                     "tag": IDENTIFIER,
                                      "str": "'%s'" % (_name)}
 
         children = core.get_children(node["name"], annotated)
         for child in children:
-            _node_type = child["code"]["type"]
-            if _node_type == VALUE:
+            _node_tag = child["code"]["tag"]
+            if _node_tag == VALUE:
                 continue
-            elif _node_type == IDENTIFIER:
+            elif _node_tag == IDENTIFIER:
                 _value = child["code"]["str"]
-            elif _node_type == FUNCTION:
+            elif _node_tag == FUNCTION:
                 _value = child["code"]["return_symbol"]
             args = {**args,
                     **{child["value"]: {"key": child["value"],
                                         "value": _value,
                                         "ref": child["name"],
-                                        "type": child["code"]["type"],
+                                        "tag": child["code"]["tag"],
                                         "str": child["code"]["str"]}}}
         fn = node["value"]
         node = cache_fn(node,
@@ -520,7 +525,7 @@ class Initializer(param_count.Hyperparam):
                         args,
                         return_symbol,
                         _name,
-                        node_type)
+                        node_tag)
 
         return node
 
@@ -532,23 +537,25 @@ class TrainableParams(param_count.Hyperparam):
 
     def generate_code(self, node, annotated, engine):
         node = deepcopy(node)
-        node_type = get_node_type(node)
+        node_tag = get_node_tag(node)
         args = dict()
         _name = "%s/%s" % (core.get_parent(node["name"],
                                            annotated,
                                            "name"),
                            node["value"])
         return_symbol = _name_strs_to_names(_name)
-        node_type = FUNCTION
+        node_tag = FUNCTION
 
         ingredient = core.get_parent(node["name"],
-                                     annotated,
-                                     "value")
-        is_trainable = const.PARAMS[ingredient][node["value"]]["derivative"]
+                                     annotated)
+        if ingredient["meta"]["trainable"] is not None:
+            is_trainable = ingredient["meta"]["trainable"]
+        else:
+            is_trainable = registry.PARAMS[ingredient["value"]][node["value"]]["derivative"]
         args["is_trainable"] = {"key": "is_trainable",
                                 "value": is_trainable,
                                 "ref": None,
-                                "type": IDENTIFIER,
+                                "tag": IDENTIFIER,
                                 "str": "%s" % str(is_trainable)}
         fn = "%s_%s" % (core.get_parent(node["name"],
                                         annotated,
@@ -558,31 +565,31 @@ class TrainableParams(param_count.Hyperparam):
         args[const.ALEX_ARG_NAME] = {"key": const.ALEX_ARG_NAME,
                                      "value": _name_strs_to_names(_name),
                                      "ref": None,
-                                     "type": IDENTIFIER,
+                                     "tag": IDENTIFIER,
                                      "str": "'%s'" % (_name)}
         children = core.get_children(node["name"], annotated)
         for child in children:
             if "code" not in child:
                 continue
-            _node_type = child["code"]["type"]
-            if _node_type == VALUE:
+            _node_tag = child["code"]["tag"]
+            if _node_tag == VALUE:
                 continue
-            elif _node_type == IDENTIFIER:
+            elif _node_tag == IDENTIFIER:
                 _value = child["code"]["str"]
-            elif _node_type == FUNCTION:
+            elif _node_tag == FUNCTION:
                 _value = child["code"]["return_symbol"]
             args = {**args,
                     **{child["value"]: {"key": child["value"],
                                         "value": _value,
                                         "ref": child["name"],
-                                        "type": child["code"]["type"],
+                                        "tag": child["code"]["tag"],
                                         "str": child["code"]["str"]}}}
         node = cache_fn(node=node,
                         fn=fn,
                         args=args,
                         return_symbol=return_symbol,
                         name=_name,
-                        node_type=node_type,
+                        node_tag=node_tag,
                         inline=False,
                         extra="%s['%s'] = %s\n" % (const.ALEX_ARG_TRAINABLE_PARAMS,
                                                    _name,
@@ -592,15 +599,15 @@ class TrainableParams(param_count.Hyperparam):
 
 def nodes(node):
     value = node["value"]
-    if value in const.REGULARIZERS:
+    if value in registry.REGULARIZERS:
         value = "regularizer"
-    elif value in const.ALL_INITIALIZERS:
+    elif value in registry.ALL_INITIALIZERS:
         value = "initializer_fn"
-    elif node["value"] in const.ALL_PARAMS:
+    elif node["value"] in registry.ALL_PARAMS:
         value = "trainable_params"
-    elif node["value"] in const.INGREDIENT_TYPES:
+    elif node["value"] in registry.INGREDIENT_TYPES:
         value = "ingredient"
-    elif node["value"] in const.SCHEDULER_BLOCK:
+    elif node["value"] in registry.SCHEDULER_BLOCK:
         value = FUNCTION
     _nodes = {"ingredient": Ingredient,
               "recipe": Recipe,
@@ -613,7 +620,7 @@ def nodes(node):
               IDENTIFIER: Identifier}
     if value not in _nodes:
         if node["type"] == "hyperparam":
-            value = get_node_type(node)
+            value = get_node_tag(node)
         else:
             value = node["type"]
     return _nodes[value](node["value"])
@@ -624,7 +631,8 @@ def generate_python(output_file,
                     engine,
                     dirname=const.CACHE_BASE_PATH,
                     load_ckpt=None,
-                    save_ckpt=None):
+                    save_ckpt=None,
+                    def_only=True):
 
     if load_ckpt is None or load_ckpt[1] is None:
         load_from_ckpt = False
@@ -641,35 +649,35 @@ def generate_python(output_file,
                                engine,
                                ckpt,
                                load_from_ckpt=load_from_ckpt,
-                               dirname=const.CACHE_BASE_PATH)()
+                               dirname=dirname)()
 
     component_str = ModelCodeBlock(output_file,
                                    config_path,
                                    engine,
                                    ckpt,
                                    load_from_ckpt=load_from_ckpt,
-                                   dirname=const.CACHE_BASE_PATH)()
+                                   dirname=dirname)()
 
     loss_str = LossCodeBlock(output_file,
                              config_path,
                              engine,
                              ckpt,
                              load_from_ckpt=load_from_ckpt,
-                             dirname=const.CACHE_BASE_PATH)()
+                             dirname=dirname)()
 
     optimizer_str = OptimizerCodeBlock(output_file,
                                        config_path,
                                        engine,
                                        ckpt,
                                        load_from_ckpt=load_from_ckpt,
-                                       dirname=const.CACHE_BASE_PATH)()
+                                       dirname=dirname)()
 
     scheduler_str = SchedulerCodeBlock(output_file,
                                        config_path,
                                        engine,
                                        ckpt,
                                        load_from_ckpt=load_from_ckpt,
-                                       dirname=const.CACHE_BASE_PATH)()
+                                       dirname=dirname)()
 
     boiler_str = cache_boiler_plate(engine)
 
@@ -680,6 +688,14 @@ def generate_python(output_file,
                                             scheduler_str)
     util.write_to(boiler_str, output_file)
     util.write_to(code, output_file)
+    if not def_only:
+        src_str = ""
+        src_str += NAMESPACES[engine].instantiate(config_path,
+                                                  load_ckpt,
+                                                  save_ckpt)
+        src_str += NAMESPACES[engine].validation()
+        src_str += NAMESPACES[engine].training(save_ckpt)
+        util.write_to(src_str, output_file)
 
 
 class CodeBlock(param_count.ParamCount):
@@ -717,7 +733,7 @@ class CodeBlock(param_count.ParamCount):
         self.output_file = output_file
         util.clear_file(self.cache_def_path)
 
-        self.passes = [[self.cache_shape],
+        self.passes = [[self.cache_shape_and_block],
                        [self._cache_alex_function_calls]]
 
 
@@ -777,10 +793,10 @@ class CodeBlock(param_count.ParamCount):
                 node = _node
             if not self._in_block(node):
                 return node
-            if node["code"]["type"] == FUNCTION:
+            if node["code"]["tag"] == FUNCTION:
                 _param_node = self.annotated[self.annotated[node["parent"]]["parent"]]
 
-                if node["value"] in const.ALL_INITIALIZERS \
+                if node["value"] in registry.ALL_INITIALIZERS \
                    and self.load \
                    and _param_node["parent"] in self.ckpt.matched:
                     _name = "%s/%s" % (_param_node["parent"],
@@ -846,12 +862,14 @@ class ParamCodeBlock(CodeBlock):
         return "param_block"
 
     def get_code_registry(self):
-        return {**const.ALL_PARAMS, **const.ALL_INITIALIZERS, **const.AS_TENSOR}
+        return {**registry.ALL_PARAMS,
+                **registry.ALL_INITIALIZERS,
+                **registry.AS_TENSOR}
 
     def generate_dl_code(self):
         return self.get_dl_code(fn_name="get_trainable_params",
                                 return_str="trainable_params",
-                                manual_args=["ckpt=None"], # if self.load else [],
+                                manual_args=["ckpt"],
                                 prefix="trainable_params = dict()\n")
 
 
@@ -859,10 +877,10 @@ class ParamCodeBlock(CodeBlock):
         cached = []
         for component in self.ckpt.components_list:
             _type = component["meta"]["type"]
-            if _type not in const.PARAMS or _type in cached:
+            if _type not in registry.PARAMS or _type in cached:
                 continue
-            for param in const.PARAMS[_type]:
-                shape = const.PARAMS[_type][param]["shape"]
+            for param in registry.PARAMS[_type]:
+                shape = registry.PARAMS[_type][param]["shape"]
                 src_fn = get_src_fn(shape)
                 self.inline_index_fns.append(src_fn)
 
@@ -875,7 +893,7 @@ class ParamCodeBlock(CodeBlock):
                                                  trg_fn,
                                                  trg_args_str)
                 self.translation_code.append(shape_code_str)
-                constructor = const.CONSTRUCTORS["params"]
+                constructor = registry.PARAM_CONSTRUCTORS["params"]
                 src_fn = "%s_%s" % (_type, param)
 
                 src_args_str = get_src_args_str(constructor)
@@ -901,7 +919,7 @@ class DataCodeBlock(CodeBlock):
         return "data_block"
 
     def get_code_registry(self):
-        return const.INPUT_TYPES
+        return registry.DATA_BLOCK
 
 
 class ModelCodeBlock(CodeBlock):
@@ -914,7 +932,7 @@ class ModelCodeBlock(CodeBlock):
         self._write_cache_to_file()
 
     def get_code_registry(self):
-        return {**const.MODEL_BLOCK}
+        return {**registry.MODEL_BLOCK}
 
     @staticmethod
     def get_block_name():
@@ -941,8 +959,8 @@ class LossCodeBlock(CodeBlock):
                                 manual_args=["trainable_params", "inputs"])
 
     def get_code_registry(self):
-        return {**const.LOSS_BLOCK,
-                **const.MODEL_BLOCK}
+        return {**registry.LOSS_BLOCK,
+                **registry.MODEL_BLOCK}
 
     @staticmethod
     def get_block_name():
@@ -964,11 +982,10 @@ class OptimizerCodeBlock(CodeBlock):
 
     def get_code_registry(self):
         if self.engine == "tf":
-            code_registry = {**const.MODEL_BLOCK,
-                             **const.OPTIMIZER_BLOCK,
-                             **const.SCHEDULER_BLOCK}
+            code_registry = {**registry.OPTIMIZER_BLOCK,
+                             **registry.SCHEDULER_BLOCK}
         elif self.engine == "pytorch":
-            code_registry = const.OPTIMIZER_BLOCK
+            code_registry = registry.OPTIMIZER_BLOCK
         return code_registry
 
     def get_blocks(self):
@@ -1004,7 +1021,7 @@ class SchedulerCodeBlock(CodeBlock):
         if self.engine == "tf":
             code_registry = {}
         elif self.engine == "pytorch":
-            code_registry = const.SCHEDULER_BLOCK
+            code_registry = registry.SCHEDULER_BLOCK
         return code_registry
 
     def get_blocks(self):
